--------------------------------------------------------------------------------
-- Copyright  2021								|
-- All Rights Reserved.								|
-- 										|
-- Permission to use, copy, modify, and distribute this software in any way	|
-- is not granted without expressed written agreement. This copyright applies	|
-- to the all code, data and artwork related to this project and all associated	|
-- projects published by the author.						|
--										|
-- More information on copyright is available from sources on-line.		|
-- Use an actual factual one!							|
--------------------------------------------------------------------------------

local AddonName, NS = ...
local L = NS.TEXTS

local Initialised, ChatAnchorFrame, ChatEventFrame
local ChatFont = NS.SharedMediaFont
local ChatFontSize = NS.FontSize + 1
local EditFontSize = NS.FontSize + 1

local ConfigureQuickSelectButtons

if NS:IsValidVersion(NS.WOW_Project.Classic) then
	ChatFontSize = ChatFontSize + 1
	EditFontSize = EditFontSize + 1
end

local FontSizes = {
	chat = {
		ChatFontSize, -- Normal
		ChatFontSize-2, -- Eagle Eye
		ChatFontSize+2, -- Large
		ChatFontSize+4, -- Magoo
	},
	edit = {
		EditFontSize, -- Normal
		EditFontSize-2, -- Eagle Eye
		EditFontSize+2, -- Large
		EditFontSize+4, -- Magoo
	},
}

local function QuickSelectButtonOnOnEnter(self)
	local parent = self:GetParent()
	parent:GetScript("OnEnter")(parent)
end

local function QuickSelectButtonOnOnLeave(self)
	local parent = self:GetParent()
	parent:GetScript("OnLeave")(parent)
end

local function QuickSelectButtonOnOnMouseDown(self)
	self.Text:SetPoint("CENTER", 2, -2)
end

local function QuickSelectButtonOnOnMouseUp(self)
	self.Text:SetPoint("CENTER", 0, 0)
end

local function QuickSelectButtonOnClick(self)
	self:GetParent():SetClicked(self:GetID())
end
local function StopLock(self)
	self.LockIcon.aglock:Stop()
	self.LockIcon:SetAlpha(0)
end

local function ChatSelectBarOnEnter(self)
	StopLock(self)
	if self.DoHide then
		self.ag:Stop()
		self:SetAlpha(1)
	end
	for i=1, #self.Buttons do
		if not self.Buttons[i].Ignore then
			self.Buttons[i]:Show()
		end
	end
end

local function SetChatLockIcon(self)
	local Parent = self or NS:GetFrame("Chat", "Parent")
	if not self:IsMouseOver() and self.LockActive and self.ActiveID and self.ActiveID ~= 1 then  -- GetMouseFocus() instead of IsMouseOver maybe???
		C_Timer.After(0.2, function()
			if not Parent:IsMouseOver() then  -- GetMouseFocus() instead of IsMouseOver maybe???
				self.LockIcon.aglock:Play()
			end
		end)
	end
end

local function ChatSelectBarOnLeave(self)
	if self:IsMouseOver() then return end  -- GetMouseFocus() instead of IsMouseOver maybe???
	if self.DoHide then
		self.ag:Play()
		SetChatLockIcon(self)
	end
end

local function SetActiveChannel(button, id, active, suppress)
	local frame = _G["ChatFrame"..id]
	if not suppress then
		PlaySound(SOUNDKIT.U_CHAT_SCROLL_BUTTON, "LeftButton")
	end
	if active then
		SetChatWindowShown(id, true)
		frame:Show()
		button.Text:SetTextColor(0, 1, 0)
	else
		SetChatWindowShown(id, false)
		frame:Hide()
		button.Text:SetTextColor(1, 1, 1)
	end
end

local function SetChannelLock()
	local Settings = NS:GetSettings("Chat")
	local Parent = NS:GetFrame("Chat", "Parent")
	Parent.LockActive = Settings.lockactive
	if not Parent.LockActive then
		StopLock(Parent)
	end
	SetChatLockIcon(Parent)
end
NS:AddOption("SetChannelLock", SetChannelLock)

local ButtonPool = {}
local function FreeQuickSelectButtons(inuse)
	for i=1, #inuse do
		tinsert(ButtonPool, inuse[i])
	end
	for k, v in pairs(ButtonPool) do
		SetChatWindowShown(v:GetID(), false)
		v:ClearAllPoints()
		v:SetPoint("BOTTOM", UIParent, "TOP", 0, 20)
	end
end

local function CreateQuickSelectButton(parent, id)
	local f 
	if #ButtonPool > 0 then
		f = ButtonPool[1]
		tremove(ButtonPool, 1)
	else
		f = CreateFrame("Button", nil, parent or UIParent)
		f.Text = f:CreateFontString()
		f.Text:SetFont(NS.Font, 11, "")
		f.Text:SetJustifyH("CENTER")
		f.Text:SetJustifyV("CENTER")
		f.Text:SetPoint("CENTER")
		f.Overlay = f:CreateTexture()
		f.Overlay:SetPoint("TOPLEFT", NS:IsValidVersion(NS.WOW_Project.Classic) and -3 or 0, 3)
		f.Overlay:SetPoint("BOTTOMRIGHT", NS:IsValidVersion(NS.WOW_Project.Classic) and 3 or 0, -3)
		f.Overlay:Hide()
		f.Overlay:SetVertexColor(0.7, 0.7, 0.7)
		f.Overlay:SetTexture(NS.TexturePath.."ChatSelectorOverlay")
		f.Overlay:SetAlpha(0.5)
		f:SetHighlightTexture(f.Overlay)
		f:SetScript("OnEnter", QuickSelectButtonOnOnEnter)
		f:SetScript("OnLeave", QuickSelectButtonOnOnLeave)
		f:SetScript("OnMouseDown", QuickSelectButtonOnOnMouseDown)
		f:SetScript("OnMouseUp", QuickSelectButtonOnOnMouseUp)
		f:SetScript("OnClick", QuickSelectButtonOnClick)
		function f:SetText(text)
			self.Text:SetText(text)
		end
		f:SetScale(NS:IsValidVersion(NS.WOW_Project.Classic) and 0.9 or 1)
	end
	local h = parent:GetHeight()
	local w = 60 
	if NS:IsValidVersion(NS.WOW_Project.Classic) then
		h = h - 4
		w = 45
	end
	f:SetSize(w, h)
	f:SetID(id)
	tinsert(parent.Buttons, f)
	parent.IDs[id] = f
	return f
end

local function AutoGuildUpdate()
	local Settings = NS:GetSettings("Chat")
	if not Settings.autoguildchannel then
		return
	end
	local Channel, Ignore
	for i=1, #Settings.channelbuttons do
		if Settings.channelbuttons[i].id == 5 then
			Channel = Settings.channelbuttons[i]
			Ignore = Channel.ignore
			break
		end
	end
	if not IsInGuild() then
		Channel.ignore=true
	else
		Channel.ignore=nil
	end
	if Channel.ignore ~= Ignore then
		ConfigureQuickSelectButtons(Settings)
	end 
end
NS:AddOption("AutoGuildUpdate", AutoGuildUpdate)

local function IsDisabledChannel(channelname, ...)-- Chaned to use channel names rather than numbers
	local Chan1 = ...			  -- (it's possible they might changedepending on order added to a ChatFrame???)
	if not Chan1 then 
		return true
	end
	for i=1, select("#", ...), 3 do
		local Chan = select(i, ...)
--		if Chan == channelnum then
		if Chan + 1 == channelname then
			local RetVal = select(i + 2, ...) -- i + 2 = enabled or disabled
			return RetVal
		end
	end
end
NS:AddOption("IsDisabledChannel", IsDisabledChannel)

local function ConfigureDelayChannels()
	local Settings = NS:GetSettings("Chat")
	if not Settings.delaychannelupdate then
		return
	end
	for action, list in pairs(Settings.delaychannelupdate) do
		for i=#list, 1, -1  do
			local Disabled = IsDisabledChannel(list[i].channelnum, GetChannelList())
			if not Disabled then
				local f = _G["ChatFrame" .. list[i].id]
				if action == "add" then
					ChatFrame_AddChannel(f, list[i].channelname)
				else
					ChatFrame_RemoveChannel(f, list[i].channelname)
				end
				tremove(list, i)
			end
		end
		if #list == 0 then
			Settings.delaychannelupdate[action] = nil
		end
	end
	if not next(Settings.delaychannelupdate) then
		Settings.delaychannelupdate = nil
		NS:UnregisterEvent("CHANNEL_UI_UPDATE", ChatEventFrame, ConfigureDelayChannels)
	end
end

local function SetChatChannelsDelayed()
	local Settings = NS:GetSettings("Chat")
	if Settings.delaychannelupdate ~= nil then
		ChatEventFrame = not ChatEventFrame and NS:GetFrame("Base", 2)
		NS:RegisterEvent("CHANNEL_UI_UPDATE", ChatEventFrame, ConfigureDelayChannels)
	end
end
NS:AddOption("SetChatChannelsDelayed", SetChatChannelsDelayed)

local function ShowQuickSelectFrames()
	local Parent = NS:GetFrame("Chat", "Parent", true)
	SetChatWindowShown(2, true)
	local Settings = NS:GetSettings("Chat").channelbuttons
	for k, v in pairs(Settings) do
		SetChatWindowShown(v.id, true)
	end
end

local function SaveChatOptions()
	local Settings = NS:GetSettings("Chat")
	if NS:IsValidVersion(WOW_PROJECT_MAINLINE) then
		Settings.whispermode = GetCVar("whisperMode") -- Saved here for accurate CopySettings.
	end
	ShowQuickSelectFrames()
	local Name = GetChatWindowInfo(1) -- Preserve information for CopysSettings
	Settings.chatgeneral = Name
end

local function CreateQuickSelectParent()
	local f = CreateFrame("Frame", nil, UIParent)
	NS:AddFrame("Chat", "Parent", f)
	ChatAnchorFrame = f
	NS:RegisterEvent("PLAYER_LOGOUT", f, SaveChatOptions)
	
	f.LockIcon = CreateFrame("Frame", nil, NS:GetFrame("Base", 1))
	local IconSize = 14
	if NS:IsValidVersion(NS.WOW_Project.Classic) then
		IconSize = 10
	end
	f.LockIcon:SetSize(IconSize, IconSize)
	f.LockIcon:SetPoint("RIGHT", f, "LEFT", 6, 0)
	f.LockIcon.Texture = f.LockIcon:CreateTexture()
	f.LockIcon.Texture:SetAllPoints()
	f.LockIcon.Texture:SetTexture(NS.TexturePath.."Lock")
	f.LockIcon:SetAlpha(0)
	
	f.Buttons = {}
	f.IDs = {}
	f:SetSize(10, 11)
	f:SetFrameStrata("MEDIUM")
	function f:SetClicked(id, suppress)
		self.ActiveID = id or self.ActiveID or 1
		for i=1, #self.Buttons do
			local Button = self.Buttons[i]
			local Channel = Button:GetID()
			SetActiveChannel(Button, Channel, Channel == self.ActiveID, suppress)
		end
	end
	f:SetScript("OnEnter", ChatSelectBarOnEnter)
	f:SetScript("OnLeave", ChatSelectBarOnLeave)
	f.ag = f:CreateAnimationGroup()
	f.ag:SetLooping("NONE")
	f.ag.parent = f
	local anim = f.ag:CreateAnimation("Alpha")
	f.anim = anim
	f.anim:SetOrder(1)
	f.anim:SetDuration(1.5)
	f.anim:SetFromAlpha(1)
	f.anim:SetToAlpha(0)

	f.LockIcon.aglock = f.LockIcon:CreateAnimationGroup()
	f.LockIcon.aglock:SetLooping("NONE")
	f.LockIcon.aglock:SetToFinalAlpha(true)
	local animlock = f.LockIcon.aglock:CreateAnimation("Alpha")
	f.LockIcon.animlock = animlock
	f.LockIcon.animlock:SetOrder(1)
	f.LockIcon.animlock:SetDuration(1.5)
	f.LockIcon.animlock:SetFromAlpha(0)
	f.LockIcon.animlock:SetToAlpha(1)

	SetChannelLock()

	f.ag:SetScript("OnFinished", function(self)
		local Parent = self:GetParent()
		for i=1, #Parent.Buttons do
			Parent.Buttons[i]:Hide()
		end
	end)
	NS:RegisterEvent("PLAYER_GUILD_UPDATE", f, AutoGuildUpdate)

	return f
end

local function ChatSelectBarSetForHide()
	local self = NS:GetFrame("Chat", "Parent", true)
	self.DoHide = NS:GetSettings("Chat").hide
	self:GetScript("OnEnter")(self)
	self:GetScript("OnLeave")(self)
end
NS:AddOption("ChatSelectBarSetForHide", ChatSelectBarSetForHide)

ConfigureQuickSelectButtons = function(chatsettings)
	local Settings = chatsettings.channelbuttons
	local Parent = NS:GetFrame("Chat", "Parent", true)
	if not Parent then 
		Parent = CreateQuickSelectParent()
	end
	FreeQuickSelectButtons(Parent.Buttons)
	if #Parent.Buttons > 0 then
		table.wipe(Parent.IDs)
		table.wipe(Parent.Buttons)
	end
	local General = CreateQuickSelectButton(Parent, 1)
	ChatSelectBarSetForHide()
	Parent:ClearAllPoints()
	if chatsettings.altpos then
		Parent:SetPoint("BOTTOMLEFT", NS:GetFrame("HotSpots", 1):GetRight()+5, 0)
	else
		Parent:SetPoint("TOPLEFT", NS:GetFrame("HotSpots", 4), "BOTTOMRIGHT", 0, NS:IsValidVersion(NS.WOW_Project.Classic) and -3 or -6)
	end
	Parent.IDs[1]:SetText(GetChatWindowInfo(1))
	for i=1, #Settings do -- 1 (General) is a given and not saved to settings, all others are optional
		if not Parent.IDs[Settings[i].id] then
			CreateQuickSelectButton(Parent, Settings[i].id)
		end
		local Channel = Parent.IDs[Settings[i].id]
		Channel:SetText(GetChatWindowInfo(Channel:GetID()))
		Parent.IDs[Settings[i].id].Ignore = Settings[i].ignore
	end
	local Last
	for i=1, #Parent.Buttons do
		local Button = Parent.Buttons[i]
		local ButtonDirection = "LeftButtonUp"
		if not chatsettings.button then
			if GetCVarBool("ActionButtonUseKeyDown") then
				ButtonDirection = "LeftButtonDown"
			end
		else
			ButtonDirection = chatsettings.button
		end
		Button:RegisterForClicks(ButtonDirection)
		if Button:GetParent().DoHide or Button.Ignore then
			Button:Hide()
		else
			Button:Show()
		end
		Button:ClearAllPoints()
		if i == 1 then
			Button:SetPoint("LEFT", 5, 0)
			Last = Button
		else
			if Button.Ignore then
				Button:SetPoint("BOTTOM", UIParent, "TOP", 0, 20)
			else
				Button:SetPoint("LEFT", Last, "RIGHT", 0, 0)
				Last = Button
			end
		end
	end
	Parent:SetWidth((Last:GetRight()-Parent.Buttons[1]:GetLeft())+10)
	Parent:SetClicked(1)
end

-- Next two functions need some cleaning up
local function SetFonts(default, highchannel, settings)
	local CFont, CFontSize = ChatFont, ChatFontSize
	local EFontSize = EditFontSize
	if not default then
		CFont, CFontSize = settings.chatfont.font, FontSizes.chat[settings.chatfont.size]
		EFontSize = FontSizes.edit[settings.editfont.size]
	end
	local LSM = LibStub("LibSharedMedia-3.0")
	CFont = LSM:Fetch("font", CFont)
	if not CFont then
		CFont = NS.Font
	end
	ChatFrame1.editBox:SetFont(CFont, EFontSize, "")
	ChatFrame1EditBoxHeader:SetFont(ChatFrame1.editBox:GetFont(), EFontSize, "")
	ChatFrame1.editBox.StdHeight = ChatFrame1.editBox:GetHeight()
	for i=1, highchannel do
-- Set ChatWindow Fonts/Sizes here based on Settings
		local Size = CFontSize
		if i == 3 or i == 4 then -- Combat Log/Loot
			Size = Size - 1
		end
		local f = _G["ChatFrame"..i]
		f:SetFont(CFont, Size, "")
	end	
	local Parent = NS:GetFrame("Chat", "Parent")
	for i=1, #Parent.Buttons do
		Parent.Buttons[i].Text:SetFont(NS.Font, CFontSize, "")
	end
end

local function ConfigureChatSettings(self, default, initprat)
	local Settings = NS:GetSettings("Chat")
	if not Settings.channelbuttons then -- Default have not been configuired, waiting on CHANNEL_UI_UPDATE
		NS:RegisterForInit("CHANNEL_UI_UPDATE", "ConfigureChatSettings", 2, ConfigureChatSettings)
		return
	end
	local HighChannel = 0
	for i=1, #Settings.channelbuttons do
		if Settings.channelbuttons[i].id > HighChannel then
			HighChannel = Settings.channelbuttons[i].id
		end
	end
	ConfigureQuickSelectButtons(Settings)
	local Prat = NS:GetSettings("Addons")["Prat-3.0"]
	if Prat and Prat.inactive then 
		SetFonts(default, HighChannel, Settings)
		return 
	end
	local CFont, CFontSize = ChatFont, ChatFontSize
	local EFontSize = EditFontSize
	if not default then
		CFont, CFontSize = Settings.chatfont.font, FontSizes.chat[Settings.chatfont.size]
		EFontSize = FontSizes.edit[Settings.editfont.size]
	end
	if Prat3DB then
		local Profile = Prat3DB.profileKeys[UnitName("player").." - "..GetRealmName()]
		if not Profile or not Prat3DB.namespaces.Prat_Editbox.profiles[Profile] then return end
		Prat3DB.namespaces.Prat_Editbox.profiles[Profile].font = CFont
		local PratSettings = Prat3DB.namespaces.Prat_Font.profiles[Profile]
		PratSettings.fontface = CFont
		for i=1, HighChannel do
			local Size = CFontSize
			if i == 3 or i ==4 then
				Size = Size - 1
			end
			PratSettings.size["ChatFrame"..i] = Size
		end
		if initprat then
			local Prat = LibStub:GetLibrary("AceAddon-3.0", true)
			Prat = Prat:GetAddon("Prat", true)
--			Prat.db:SetProfile(Profile)
			Prat:UpdateProfile()
		end
	end
	ChatFrame1.editBox:SetFont(ChatFrame1.editBox:GetFont(), EFontSize, "")
	ChatFrame1EditBoxHeader:SetFont(ChatFrame1.editBox:GetFont(), EFontSize, "")
	ChatFrame1.editBox.StdHeight = ChatFrame1.editBox:GetHeight()
end
-- after check for Prat IsLoaded
NS:RegisterForInit("PLAYER_LOGIN", "ConfigureChatSettings", 4, ConfigureChatSettings)
NS:AddOption("ConfigureChatSettings", ConfigureChatSettings)
              
local function SetChatFrameManaged(self, managed, name)
	if not self.FMSaved then
		self.FMSaved = true
		NS:SavePoints(self)
		if self.editBox and not self.editBox.FMSaved then
			self.editBox.FMSaved = true
			NS:SavePoints(self.editBox)
		end
		self.RectInsets = { self:GetClampRectInsets() }
		self.OldBackground = self.Background:GetTexture()
		if self.editBox.focusLeft then  -- Non retail test
			self.editBox.OldfocusLeft = self.editBox.focusLeft:GetTexture()
			self.editBox.OldfocusMid = self.editBox.focusMid:GetTexture()
			self.editBox.OldfocusRight = self.editBox.focusRight:GetTexture()
		end
		self:SetMaxLines(400)
	end
	self:SetScript("OnHide", managed and nil or self.OldOnHide)
	_G[self:GetName().."RightTexture"]:SetShown(not managed)
	_G[self:GetName().."BottomTexture"]:SetShown(not managed)
	local tab = _G[self:GetName().."Tab"]
	NS:DisappearFrame(tab, not managed)
	if managed then
		self:SetClampRectInsets(0, 0, 0, 0)
	else
		self:SetClampRectInsets(unpack(self.RectInsets))
		tab:SetText(name)
		PanelTemplates_TabResize(tab, tab.sizePadding or 0)
-- DragonFlight Removed		tab.textWidth = _G[tab:GetName().."Text"]:GetWidth()
	end
	self.Background:SetTexture(managed and nil or self.OldBackground)
	self.Background:SetShown(not managed)
	self:SetFading(not managed)
	NS:DisappearFrame(self.buttonFrame, not managed)
	_G[self:GetName().."TopTexture"]:SetShown(not managed)
	_G[self.editBox:GetName().."Left"]:SetShown(not managed)
	_G[self.editBox:GetName().."Mid"]:SetShown(not managed)
	_G[self.editBox:GetName().."Right"]:SetShown(not managed)
	if self.editBox.focusLeft then  -- Non retail test
		self.editBox.focusLeft:SetTexture(managed and nil or self.editBox.OldfocusLeft)
		self.editBox.focusMid:SetTexture(managed and nil or self.editBox.OldfocusMid)
		self.editBox.focusRight:SetTexture(managed and nil or self.editBox.OldfocusRight)
	end
end

local BorderWidth, ChatWidth, EditBoxHooked, LogAnchor
local function SizeWithEdit(self)
	NS.PointFrame.SetPoint(ChatFrame1, "TOPLEFT", NS:GetFrame("HotSpots", 2), "CENTER", -BorderWidth + 10, -(self:GetHeight() + 5))
end

local function HookCF1(self)
--[[
	NS.PointFrame.ClearAllPoints(self.editBox)
	NS.PointFrame.SetPoint(self.editBox, "TOPLEFT", NS:GetFrame("HotSpots", 2), "CENTER", -BorderWidth, -2)
	NS.PointFrame.SetPoint(self.editBox, "TOPRIGHT", NS:GetFrame("HotSpots", 3), "CENTER", BorderWidth, -2)
	NS.PointFrame.ClearAllPoints(self)
	NS.PointFrame.SetPoint(self, "TOPLEFT", self.editBox, "BOTTOMLEFT", 10, 0)
	local w = ChatFrame1.ScrollBar and ChatFrame1.ScrollBar:GetWidth() or 0
	NS.PointFrame.SetPoint(self, "BOTTOMRIGHT", NS:GetFrame("HotSpots", 1), "CENTER", self.editBox:GetWidth() - (w + (ChatWidth * 2)), 4)
]]--
	NS.PointFrame.ClearAllPoints(self.editBox)
	NS.PointFrame.SetPoint(self.editBox, "TOPLEFT", NS:GetFrame("HotSpots", 2), "CENTER", -BorderWidth, -2)
	NS.PointFrame.SetPoint(self.editBox, "TOPRIGHT", NS:GetFrame("HotSpots", 3), "CENTER", BorderWidth, -2)
	NS.PointFrame.ClearAllPoints(self)
--	NS.PointFrame.SetPoint(self, "TOPLEFT", NS:GetFrame("HotSpots", 2), "CENTER", -BorderWidth + 10, -self.editBox:GetHeight())
	SizeWithEdit(self.editBox)
	local w = ChatFrame1.ScrollBar and ChatFrame1.ScrollBar:GetWidth() or 0
	NS.PointFrame.SetPoint(self, "BOTTOMRIGHT", NS:GetFrame("HotSpots", 1), "CENTER", self.editBox:GetWidth() - (w + (ChatWidth * 2)), 4)
	if not EditBoxHooked then
		EditBoxHooked = true
		self.editBox:HookScript("OnSizeChanged", SizeWithEdit)
		self.editBox:SetMultiLine(NS:GetSettings("Chat").multiline)
	end
end
local function HookCF2(self)
	NS.PointFrame.ClearAllPoints(self)
	NS.PointFrame.SetPoint(self, "TOPLEFT", LogAnchor, 3, 0)
	NS.PointFrame.SetPoint(self, "BOTTOMRIGHT", LogAnchor, "BOTTOM", -19, 0)
end
local function HookCF3(self)
	NS.PointFrame.ClearAllPoints(self)
	NS.PointFrame.SetPoint(self, "TOPRIGHT", UIParent, "BOTTOMLEFT", -20, -20)
end
local function HookCF4(self)
	NS.PointFrame.ClearAllPoints(self)
	NS.PointFrame.SetPoint(self, "TOPRIGHT", LogAnchor, -12, 0)
	NS.PointFrame.SetPoint(self, "BOTTOMLEFT", LogAnchor, "BOTTOM", 3, 0)
end
local function HookCFOther(self)
	NS.PointFrame.ClearAllPoints(self)
	if self.FMManaged then
		NS:SavePoints(self, true)
	end
	NS.PointFrame.SetAllPoints(self, ChatFrame1)
end

local function ConfigureChatFrames()
	local Settings = NS:GetSettings("Chat").channelbuttons
	if not Settings then -- Default have not been configuired, waiting on CHANNEL_UI_UPDATE
		NS:RegisterForInit("CHANNEL_UI_UPDATE", "ConfigureChatFrames", 3, ConfigureChatFrames)
		return
	end
	-- "Managed" frames have buttons. 
	-- "Unmanaged" frames saved in NS:GetSettings("Chat").channelbuttons are for CopySettings only (copy channel name)
	local Managed = {
		[1] = true,
		[2] = true,
		[3] = true,
		[4] = true,
	}
	local Names = {}
	for k, v in pairs(Settings) do
		if not v.ignore then
			Managed[v.id] = not v.ignore   
		end
		Names[v.id] = v.name
--		if v.id < 9 then
--			Managed[v.id] = true
--		else
--			Managed[v.id] = not v.unmanaged
--		end
	end
	LogAnchor = NS:GetFrame("Utility", "Logs")
	BorderWidth = NS:GetOption("Metrics").BorderWidth * 1.4
	ChatWidth = BorderWidth + 2
	local Start = Initialised and 5 or 1
	Initialised = true
	for i=Start, NUM_CHAT_WINDOWS do
		local f = _G["ChatFrame"..i]
		if f then
			SetChatFrameManaged(f, Managed[i], Names[i])
			if f.ResizeButton then
				f.ResizeButton:Hide()
			end
			if i == 1 then
				f.editBox:SetHistoryLines(1)
				f.editBox.History = {}
				f.editBox.HistoryIndex = 0
				f.editBox:SetAltArrowKeyMode(nil)
				hooksecurefunc(f, "SetPoint", HookCF1)
				HookCF1(f)
--			elseif i == 2 then
			elseif i == 3 then
				hooksecurefunc(f, "SetPoint", HookCF3)
				HookCF3(f)
				f:Hide()
--			elseif i == 3 then
			elseif i == 2 then
				f:SetParent(LogAnchor)
				hooksecurefunc(f, "SetPoint", HookCF2)
				HookCF2(f)
				SetChatWindowSavedDimensions(f:GetID(), f:GetWidth(), f:GetHeight()) 
			elseif i == 4 then
				f:SetParent(LogAnchor)
				hooksecurefunc(f, "SetPoint", HookCF4)
				HookCF4(f)
				SetChatWindowSavedDimensions(f:GetID(), f:GetWidth(), f:GetHeight()) 
--			elseif i > 4 then --i < 9 then
			else -- 5-10
--				NS:SavePoints(f)
				hooksecurefunc(f, "SetPoint", HookCFOther)
				f.FMManaged = Managed[i]
				HookCFOther(f)
		
--			else
--				f:OldClearAllPoints()
--				f:SetAllPoints(ChatFrame1)
--				FCF_SetWindowAlpha(f, 0)
			end
		end
	end
end
NS:AddOption("ConfigureChatFrames", ConfigureChatFrames)

local CF1_ShowAction, Reset
local TabList = {}
local function ResetTabList(...)
	table.wipe(TabList)
	local tab
	for i=1, select("#", ...) do
		tab = select(i, ...)
		tinsert(TabList, tab)
	end
end

local function ConfigureChatAnchor()
	NS:AddToLOD("Blizzard_Communities", function()
		CommunitiesFrame:HookScript("OnShow", function()
			ShowQuickSelectFrames()
		end)
		CommunitiesFrame:HookScript("OnHide", function()
			SetChatWindowShown(2, false)
			NS:GetFrame("Chat", "Parent"):SetClicked()
		end)
	end)
	ChatConfigFrame.ChatTabManager:HookScript("OnShow", function(self)
		ShowQuickSelectFrames()
		if not Reset then
			Reset = true
			self:GetScript("OnShow")(self)
			return
		else
			Reset = nil
		end
		if WOW_PROJECT_ID ~= WOW_PROJECT_MAINLINE then
			ResetTabList(self:GetChildren())
			for i = 1, #TabList do
				local tab = TabList[i]
				local ChatAlpha = tab:GetAlpha() 
				local id = tab:GetID()
--				if id == 2 or id == 3 then 
				if id == 3 then 
					tab:SetEnabled(false) 
					tab:SetAlpha(0.3) 
				else 
					tab:SetEnabled(true) 
					tab:SetAlpha(ChatAlpha) 
				end 
			end
		end
	end)
	ChatConfigFrame:HookScript("OnHide", function(self)
		SetChatWindowShown(2, false)
		NS:GetFrame("Chat", "Parent"):SetClicked()
	end)

	ChatConfigFrameDefaultButton:HookScript("OnShow", function(self)
		self:Hide()
	end)
	ChatConfigFrameRedockButton:HookScript("OnShow", function(self)
		self:Hide()
	end)
	ChatConfigFrameDefaultButton:Hide()
	ChatConfigFrameRedockButton:Hide()
--	if ChatFrame1.editBox.History then return end
	NS:DisappearFrame(GeneralDockManager)
	if QuickJoinToastButton then -- Non retail test
	 	NS:DisappearFrame(QuickJoinToastButton)
	end
	if CombatLogQuickButtonFrame then -- Non retail test I think
		NS:DisappearFrame(CombatLogQuickButtonFrame)
		NS:DisappearFrame(CombatLogQuickButtonFrame_Custom)
	end
	NS:DisappearFrame(ChatFrameChannelButton)
	NS:DisappearFrame(ChatFrameMenuButton)
	ConfigureChatFrames()

	ChatFrame1:HookScript("OnHide", function(self)
		CF1_ShowAction = nil
	end)

	ChatFrame1:HookScript("OnShow", function(self)
		local Chan = ChatAnchorFrame
		if not Chan then return end
		if Chan.LockActive and Chan.ActiveID then
			Chan:SetClicked(Chan.ActiveID, true)
		else
			if not CF1_ShowAction then
				CF1_ShowAction = true
				Chan:SetClicked(1, true)
			end
		end
	end)
	SetChatChannelsDelayed()
--	SetFonts(default)
end

local function DelayAnchor()
	C_Timer.After(12, ConfigureChatAnchor)
end
NS:RegisterForInit("PLAYER_ENTERING_WORLD", "ConfigureChatAnchor", 1, ConfigureChatAnchor)
--NS:RegisterForInit("PLAYER_ENTERING_WORLD", "ConfigureChatAnchor", 1, DelayAnchor)
